function [col] = run_exp(dataname)
% run `dataname' with the methods shown in Sec 2.2
% the results are stored sequentially in `col'

%datanames
correctdata = ['../data/' dataname '.correct'];
traindata = ['../data/' dataname '.train'];
neg1data = ['../data/' dataname '.neg1'];
neg1model = [dataname '.neg1.model'];
osvmsvrdata = ['../data/' dataname '.osvmsvr'];
trainmodel = [dataname '.train.model'];
osvmsvrmodel = [dataname '.osvmsvr.model'];

%learner names : XXX
libsvm = '../classifier/libsvm-infensemble-2.8-fv/';
liblinear = '../classifier/liblinear-1.51-fv/';

%The first positive data
onepos = 1;
if strcmp(dataname,'hiva');
	onepos = 1;
	osvmsvrmap = '-t 0';
	posratio = 0.0352;
elseif strcmp(dataname,'ibn_sina');
	onepos = 4;	
	osvmsvrmap = '-t 2';
	posratio = 0.3784;
elseif strcmp(dataname,'nova');
	onepos = 11;	
	osvmsvrmap = '-t 0';
	posratio = 0.2845;
elseif strcmp(dataname,'orange');
	onepos = 54;	
	osvmsvrmap = '-t 2';
	posratio = 0.0178;
elseif strcmp(dataname,'sylvaexp');
	onepos = 4;	
	osvmsvrmap = '-t 0';
	posratio = 0.0615;
elseif strcmp(dataname,'sylva');
	onepos = 4;	
	osvmsvrmap = '-t 2';
	posratio = 0.0615;
elseif strcmp(dataname,'zebra');
	onepos = 23;	
	osvmsvrmap = '-t 2';
	posratio = 0.0458;
end

%data processing
[y x] = libsvmread(correctdata);
y(:) = -1;
y(onepos) = 1;
libsvmwrite(neg1data,y,x);
libsvmwrite(traindata,[1],x(onepos,:));


%The returning column
colidx = 1;
%All negative
system([liblinear 'train -w-1 1e-11 -s 0 '  neg1data]);
system([liblinear 'predict ' correctdata ' ' neg1model ' '  dataname '.predict']);
col(colidx) = count_auc(dataname);
colidx = colidx + 1;


%OSVMs
for t=[0 2 3 6]
	param = sprintf('-s 2 -t %d',t);
	system([libsvm 'svm-train ' param ' ' traindata]);
	system([libsvm 'svm-predict ' correctdata ' ' trainmodel ' '  dataname '.predict']);
	auc = count_auc(dataname);
	col(colidx) = auc; 
	colidx = colidx + 1;
end

%OSVM + \epsilon-SVR 
system([libsvm 'svm-train -s 2 ' osvmsvrmap ' ' traindata]);
system([libsvm 'svm-predict ' correctdata ' ' trainmodel ' '  dataname '.predict']);

fvs = load([dataname '.predict']);
moreneg1s = osvmsvr_getnegative(fvs,0.1,3)';
mores = [moreneg1s onepos];
libsvmwrite(osvmsvrdata,y(mores),x(mores,:));

system([libsvm 'svm-train -s 3 ' osvmsvrmap ' ' osvmsvrdata]);
system([libsvm 'svm-predict ' correctdata ' ' osvmsvrmodel ' '  dataname '.predict']);
auc = count_auc(dataname);
col(colidx) = auc; 
colidx = colidx + 1;




%svmlin r = 0.1
y(:) = 0;
y(onepos) = 1;
size(y)
size(x)
[w fv] = svmlin('-A 2 -R 0.1',x,y);

fp = fopen([dataname '.predict'],'w');
fprintf(fp,'%.12f\n',fv);
fclose(fp);

col(colidx) = count_auc(dataname);
colidx = colidx + 1;

%svmlin r = posratio
y(:) = 0;
y(onepos) = 1;
[w fv] = svmlin(sprintf('-A 2 -R %.6f',posratio),x,y);

fp = fopen([dataname '.predict'],'w');
fprintf(fp,'%.12f\n',fv);
fclose(fp);

col(colidx) = count_auc(dataname);
colidx = colidx + 1;
%%%%%End of generating column

system(['rm ' traindata]);
system(['rm ' neg1data]);
system(['rm ' neg1data]);
system(['rm ' neg1model]);
system(['rm ' trainmodel]);
system(['rm ' osvmsvrdata]);
system(['rm ' osvmsvrmodel]);
system(['rm *.predict']);
